=== modified file 'NEWS'
--- NEWS	2005-10-04 02:47:52 +0000
+++ NEWS	2005-10-05 02:47:53 +0000
@@ -40,6 +40,9 @@
     * Removed testsweet module so that tests can be run after 
       bzr installed by 'bzr selftest'.
 
+    * 'bzr selftest' command-line arguments can now be partial ids
+      of tests to run, e.g. 'bzr selftest test_weave'
+
       
 bzr 0.0.9 2005-09-23
 

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2005-10-03 02:54:29 +0000
+++ bzrlib/builtins.py	2005-10-05 02:47:53 +0000
@@ -1101,10 +1101,20 @@
 
 
 class cmd_selftest(Command):
-    """Run internal test suite"""
+    """Run internal test suite.
+    
+    This creates temporary test directories in the working directory,
+    but not existing data is affected.  These directories are deleted
+    if the tests pass, or left behind to help in debugging if they
+    fail.
+    
+    If arguments are given, they are regular expressions that say
+    which tests should run."""
+    # TODO: --list should give a list of all available tests
     hidden = True
+    takes_args = ['testnames*']
     takes_options = ['verbose', 'pattern']
-    def run(self, verbose=False, pattern=".*"):
+    def run(self, testnames_list=None, verbose=False, pattern=".*"):
         import bzrlib.ui
         from bzrlib.selftest import selftest
         # we don't want progress meters from the tests to go to the
@@ -1114,7 +1124,9 @@
         bzrlib.trace.info('running tests...')
         try:
             bzrlib.ui.ui_factory = bzrlib.ui.SilentUIFactory()
-            result = selftest(verbose=verbose, pattern=pattern)
+            result = selftest(verbose=verbose, 
+                              pattern=pattern,
+                              testnames=testnames_list)
             if result:
                 bzrlib.trace.info('tests passed')
             else:

=== modified file 'bzrlib/selftest/__init__.py'
--- bzrlib/selftest/__init__.py	2005-10-04 05:15:51 +0000
+++ bzrlib/selftest/__init__.py	2005-10-05 02:47:53 +0000
@@ -24,6 +24,7 @@
 import errno
 import subprocess
 import shutil
+import re
 
 import bzrlib.commands
 import bzrlib.trace
@@ -112,26 +113,18 @@
         return EarlyStoppingTestResultAdapter(result)
 
 
-class filteringVisitor(TestUtil.TestVisitor):
-    """I accruse all the testCases I visit that pass a regexp filter on id
-    into my suite
-    """
-
-    def __init__(self, filter):
-        import re
-        TestUtil.TestVisitor.__init__(self)
-        self._suite=None
-        self.filter=re.compile(filter)
-
-    def suite(self):
-        """answer the suite we are building"""
-        if self._suite is None:
-            self._suite=TestUtil.TestSuite()
-        return self._suite
-
-    def visitCase(self, aCase):
-        if self.filter.match(aCase.id()):
-            self.suite().addTest(aCase)
+def iter_suite_tests(suite):
+    """Return all tests in a suite, recursing through nested suites"""
+    for item in suite._tests:
+        if isinstance(item, unittest.TestCase):
+            yield item
+        elif isinstance(item, unittest.TestSuite):
+            for r in iter_suite_tests(item):
+                yield r
+        else:
+            raise Exception('unknown object %r inside test suite %r'
+                            % (item, suite))
+
 
 class TestSkipped(Exception):
     """Indicates that a test was intentionally skipped, rather than failing."""
@@ -397,8 +390,30 @@
         ##assert 0
 
 
-
-def run_suite(suite, name='test', verbose=False, pattern=".*"):
+def filter_suite_by_re(suite, pattern):
+    result = TestUtil.TestSuite()
+    filter_re = re.compile(pattern)
+    for test in iter_suite_tests(suite):
+        if filter_re.match(test.id()):
+            result.addTest(test)
+    return result
+
+
+def filter_suite_by_names(suite, wanted_names):
+    """Return a new suite containing only selected tests.
+    
+    Names are considered to match if any name is a substring of the 
+    fully-qualified test id (i.e. the class ."""
+    result = TestSuite()
+    for test in iter_suite_tests(suite):
+        this_id = test.id()
+        for p in wanted_names:
+            if this_id.find(p) != -1:
+                result.addTest(test)
+    return result
+
+
+def run_suite(suite, name='test', verbose=False, pattern=".*", testnames=None):
     TestCaseInTempDir._TEST_NAME = name
     if verbose:
         verbosity = 2
@@ -407,9 +422,11 @@
     runner = TextTestRunner(stream=sys.stdout,
                             descriptions=0,
                             verbosity=verbosity)
-    visitor = filteringVisitor(pattern)
-    suite.visit(visitor)
-    result = runner.run(visitor.suite())
+    if testnames:
+        suite = filter_suite_by_names(suite, testnames)
+    if pattern != '.*':
+        suite = filter_suite_by_re(suite, pattern)
+    result = runner.run(suite)
     # This is still a little bogus, 
     # but only a little. Folk not using our testrunner will
     # have to delete their temp directories themselves.
@@ -421,9 +438,10 @@
     return result.wasSuccessful()
 
 
-def selftest(verbose=False, pattern=".*"):
+def selftest(verbose=False, pattern=".*", testnames=None):
     """Run the whole test suite under the enhanced runner"""
-    return run_suite(test_suite(), 'testbzr', verbose=verbose, pattern=pattern)
+    return run_suite(test_suite(), 'testbzr', verbose=verbose, pattern=pattern,
+                     testnames=testnames)
 
 
 def test_suite():

